#!/usr/bin/env python3

"""
This CLI gives us the possibility to use the Trackl API via commandline!
"""

import os
import requests
import dateutil.parser
from datetime import timezone


def _handle_error(resp):
    try:
        resp.raise_for_status()
        return resp.json()
    except:
        print(f'Error')
        raise Exception(resp.content.decode('utf8'))


def _parse_date_time(val):
    """
    Timestamps are stored as UTC in the database. The API returns them as UTC (*Z). Ideally we should store the user's timezone in the database and the API should return the timestamps with the timezone information the user has.
    Until this is done, a client needs to convert that timestamp back into it's local timezone
    """

    return dateutil.parser.isoparse(val).replace(tzinfo=timezone.utc).astimezone(tz=None) if val else None


class TracklClient:
    """
    API doc: https://trackl.labsolution.lu/apidoc
    """

    def __init__(self, api_endpoint=None, workspace=None, api_key=None, login_email=None, password=None):
        self.api_endpoint = api_endpoint or os.environ['api_endpoint']
        
        self.api_key = api_key or os.environ['api_key']
        
        # Get API Key with login
        if not self.api_key:
            workspace = workspace or os.environ.get('workspace')
            login_email = login_email or os.environ.get('login_email')
            password = password or os.environ.get('password')

            resp = requests.get(
                f'{self.api_endpoint}/auth/login',
                params={
                    'workspace_name': self.workspace,
                    'email': login_email,
                    'password': password,
                }
            )
            ret = _handle_error(resp)
            self.api_key = ret['access_api_key']

        print(f'Trackl API CLI successfully initialised')

    def session_get(self):
        # Load Session
        resp = requests.get(
            f'{self.api_endpoint}/session',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        session_info = _handle_error(resp)
        return session_info

    def teams_list(self):
        # Load Teams
        resp = requests.get(
            f'{self.api_endpoint}/teams',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        teams = _handle_error(resp)
        return teams

    def teams_search(self, name=None):
        resp = requests.get(
            f'{self.api_endpoint}/teams/search',
            headers={'Authorization': f'Bearer {self.api_key}'},
            params={
                'name': name,
            }
        )
        teams = _handle_error(resp)
        return teams

    def teams_create(self, active, name):
        resp = requests.post(
            f'{self.api_endpoint}/teams',
            headers={'Authorization': f'Bearer {self.api_key}'},
            json={
                "active": active,
                "name": name,
            },
        )
        team = _handle_error(resp)
        return team

    def users_list(self):
        # Load Users
        resp = requests.get(
            f'{self.api_endpoint}/users',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        users = _handle_error(resp)
        return users

    def users_search(self, email=None, name=None):
        resp = requests.get(
            f'{self.api_endpoint}/users/search',
            headers={'Authorization': f'Bearer {self.api_key}'},
            params={
                'email': email,
                'name': name,
            }
        )
        users = _handle_error(resp)
        return users

    def users_create(self, active, email, name, password, permissions):
        resp = requests.post(
            f'{self.api_endpoint}/users',
            headers={'Authorization': f'Bearer {self.api_key}'},
            json={
                "active": active,
                "email": email,
                "name": name,
                "password": password,
                "permissions": permissions.split(',')
            },
        )
        user = _handle_error(resp)
        return user

    def teams_add_user(self, team_id=None, user_id=None, team_name=None, user_email=None):
        # Load Team
        if team_name:
            team_id = self.teams_search(name=team_name)['items'][0]['id']

        # Load user
        if user_email:
            user_id = self.users_search(email=user_email)['items'][0]['id']

        # Do it
        resp = requests.post(
            f'{self.api_endpoint}/teams/{team_id}/users',
            headers={'Authorization': f'Bearer {self.api_key}'},
            json={
                "user_id": user_id,
            },
        )
        teamuser = _handle_error(resp)
        return teamuser

    def teams_remove_user(self, team_id=None, user_id=None, team_name=None, user_email=None):
        # Load Team
        if team_name:
            team_id = self.teams_search(name=team_name)['items'][0]['id']

        # Load user
        if user_email:
            user_id = self.users_search(email=user_email)['items'][0]['id']

        # Do it
        resp = requests.delete(
            f'{self.api_endpoint}/teams/{team_id}/users/{user_id}',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        teamuser = _handle_error(resp)
        return teamuser

    def teams_list_users(self, team_id=None, team_name=None):
        # Load Team
        if team_name:
            team_id = self.teams_search(name=team_name)['items'][0]['id']
        
        # Do it
        resp = requests.get(
            f'{self.api_endpoint}/teams/{team_id}/users',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        users = _handle_error(resp)
        return users

    def records_search(self, page=None, page_type=None, page_size=None, id=None, user_id=None, team_id=None, reference=None, redmine_issue=None, client=None, project=None, description_like=None, start_date_time_from=None, start_date_time_to=None, end_date_time_is_null=None, include_user_name=None, update_date_time_from=None, tag=None):
        resp = requests.get(
            f'{self.api_endpoint}/records/search',
            headers={'Authorization': f'Bearer {self.api_key}'},
            params={
                'page': page,
                'page_type': page_type,
                'page_size': page_size,
                'id': id,
                'user_id': user_id,
                'team_id': team_id,
                'reference': reference,
                'redmine_issue': redmine_issue,
                'client': client,
                'project': project,
                'description_like': description_like,
                'start_date_time_from': start_date_time_from,
                'start_date_time_to': start_date_time_to,
                'end_date_time_is_null': end_date_time_is_null,
                'include_user_name': include_user_name,
                'update_date_time_from': update_date_time_from,
                'tag': tag,
            }
        )
        records = _handle_error(resp)

        # Parse date times
        for record in records['items']:
            record['start_date_time'] = _parse_date_time(record['start_date_time'])
            record['end_date_time'] = _parse_date_time(record['end_date_time'])
            record['create_date_time'] = _parse_date_time(record['create_date_time'])
            record['update_date_time'] = _parse_date_time(record['update_date_time'])

        return records

    def records_validate(self, id):
        resp = requests.get(
            f'{self.api_endpoint}/records/{id}/validate',
            headers={'Authorization': f'Bearer {self.api_key}'},
        )
        record_validtiy_infos = _handle_error(resp)
        return record_validtiy_infos



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['trackl_client']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.5.0,<0.6.0', 'python-dateutil>=2.8.2,<3.0.0']

setup_kwargs = {
    'name': 'trackl-client',
    'version': '2.1.1',
    'description': 'Trackl Python Client',
    'long_description': "# Trackl Python Client\n\n## Install\n\n`python3 -m pip install git+https://gitlab.com/denny.weinberg/trackl-python-client`\nor\n`python3 -m pip install trackl-python-client`\n\n## Python\n\n**Usage/Init**\n\n```python\nfrom trackl_client import TracklClient\n\n\nclient = TracklClient(api_endpoint='http://localhost:8080/api/v1', api_key='ey...')\n\n#\xa0Or with workspace, email and password\nclient = TracklClient(api_endpoint='http://localhost:8080/api/v1', workspace='test' login_email='test@test.com', password='test')\n```\n\n**Examples**\n\n```python\n# Teams: List\nteams = client.teams_list()\n\n# Teams: Search\nteams = client.teams_search(name='Name')\n\n# Teams: Create\nteam = client.teams_create(True, 'Name')\n\n# Users: List\nusers = client.users_list()\n\n# Users: Search\nusers = client.users_search(email='test@test.com')\n\n# Users: Create\nuser = client.users_create(True, 'email', 'Name', 'testPWD123$', ['record.read', 'record.write'])\n\n# Teams: Add User\nclient.teams_add_user(team_name='Name',  user_email='test@test.com')\n\n# Teams: Remove User\nclient.teams_remove_user(team_name='Name', user_email='test@test.com')\n\n# Teams: List Users\nusers = client.teams_list_users(team_name='Office')\n\n# Records: Search\nrecords = client.records_search(page=1, page_type='elements', id='1,2', page_size=200, start_date_time_from='2022-10-01', start_date_time_to='2022-10-05', end_date_time_is_null=False, include_user_name=True)\n\n# Records: Validate\nrecord_validity_infos = client.records_validate(1)\n```\n\n## CLI\n\n**Usage**\n\n```bash\npython3 -m trackl_client.cli\n    --api_endpoint ${api_endpoint} --workspace ${workspace} --api_key ${api_key} --login_email ${login_email} --password ${password} \n    users_create|teams_create|teams_add_user|teams_remove_user\n    [--help]\n    other_arg_1 --other_kwarg_1\n\n# More:\n\n# python3 -m trackl_client.cli --help\n\n# python3 -m trackl_client.cli records_search --help\n```\n\n**Init**\n\n```bash\nexport api_endpoint='http://localhost:8080/api/v1'\n\nexport api_key='ey...'\n\n#\xa0Or with workspace, email and password\n#\xa0export workspace='test'\n#\xa0export login_email='test@test.com'\n#\xa0export password='test'\n```\n\n**Examples**\n\n```bash\n# Teams: List\npython3 -m trackl_client.cli teams_list\n\n# Teams: Search\npython3 -m trackl_client.cli teams_search --name Name\n\n# Teams: Create\npython3 -m trackl_client.cli teams_create True Name\n\n# Users: List\npython3 -m trackl_client.cli users_list\n\n# Users: Search\npython3 -m trackl_client.cli users_search --email test@test.com\n\n# Users: Create\npython3 -m trackl_client.cli users_create True email Name testPWD123$ record.read,record.write\n\n# Teams: Add User\npython3 -m trackl_client.cli teams_add_user --team_name Name --user_email test@test.com\n\n# Teams: Remove User\npython3 -m trackl_client.cli teams_remove_user --team_name Name --user_email test@test.com\n\n# Teams: List Users\npython3 -m trackl_client.cli teams_list_users --team_name Office\n\n# Records: Search\npython3 -m trackl_client.cli records_search --page 1 --page_type elements --id 1,2 --page_size 200 --start_date_time_from 2022-10-01 --start_date_time_to 2022-10-05 --end_date_time_is_null false --include_user_name True\n\n# Records: Validate\npython3 -m trackl_client.cli records_validate 1\n```\n",
    'author': 'Nobody',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
